#!perl 

# calculates divergence in windows for two fastas 

use POSIX;

if (@ARGV < 3){
	print "\nusage: perl fa2div_window.pl infile scaffold windowsize\n\n";
	exit;
}

my $infile = shift(@ARGV); chomp $infile;
open REF, $infile or die "wrong format for infile\n";

my $scaffold =  shift(@ARGV); chomp $scaffold;
my $windowsize =  shift(@ARGV); chomp $windowsize;

my %data = ();
my @sequence_names = ();
my @sequence_temp =();  
my @data_temp =();	# needed to read in nexus files

#**************** read in sequence & create output sequence
	while (my $line = <REF>) {
		chomp $line;
		$line =~ s/\s+//g; # remove all white spaces
		$line =~ s/type/\ttype/g; # insert a tab
		if ($line =~ />/) {
			(my $name, my $other) = split(/\t/, $line); chomp $name;
			push(@sequence_names, $name);
			$sequence_temporary = join ("", @data_temp);
			push(@sequence_temp,$sequence_temporary); 
			@data_temp=();
			} 
		else {push(@data_temp, $line);}
		}
	close REF;
	$sequence_temporary = join ("", @data_temp);
	push(@sequence_temp,$sequence_temporary); 
	for $x (0..(scalar(@sequence_names)-1))	# put sequences into @data
		{
		$data{$sequence_names[$x]}=$sequence_temp[$x+1];
		}

my $seq_count = scalar (keys (%data));
print "# of sequences loaded: $seq_count\n";

my @sequences=();
foreach $key (keys (%data)){ #this part assigns the two sequences for a particular scaffold to an array
	if (substr($key,0) =~ /$scaffold/) {
		print "$key\n";
		push(@sequences, $data{$key})
		}
}

open OUT, ">>divergence_ld";

$num_windows = floor(length($sequences[0])/$windowsize);

	my $div=0; my $site_counter=0;
	for $i (0..$num_windows){
	        $sub_sequence1 = uc(substr($sequences[0], $i*$windowsize, $windowsize));
		$sub_sequence2 = uc(substr($sequences[1], $i*$windowsize, $windowsize));
		for $k (0..(length($sub_sequence1)-1)){
			my $base1=substr($sub_sequence1,$k,1);
			my $base2=substr($sub_sequence2,$k,1);
			if (($base1 ne "N") && ($base2 ne "N")){
				$site_counter++;
				if ($base1 ne $base2){
					$div++;
				} # if they are different
			} # of both are non-N bases
		} # for all sites in the window
	my $midpoint= ($i+1)*$windowsize/2 + $i*$windowsize/2;
	print OUT "$scaffold\t$midpoint\t$div\t$site_counter\t", "\n";
	$div=0; $site_counter=0;
	} # for each window



