<?php
ini_set ( "memory_limit", -1 );
$sGenotypeFilePop1 = "excluded.filtered.ancestry-par1.tsv_final";
$sGenotypeFilePop2 = "excluded.filtered.ancestry-par2.tsv_final";

$nCutoffProb = 0.90;
$nScfldLimit = 196; 
$sOutPrefix = "TLMC_subset";
$sDOut= $sOutPrefix.".D.txt";
$sDprimeOut= $sOutPrefix.".Dprime.txt";
$sRsquareOut= $sOutPrefix.".R.txt"; 
$sNumIndividualOut = $sOutPrefix.".ind.counts.txt";
$sPlotScript='R --no-restore --no-save --args '.$sOutPrefix.' < plotAxis.R';
$nIndividualLimit = 1000; //limit the analysis to the first 1000 individual. change this to a big number to include all individuals! 
$nSubSampleEvery = 1; //marker subsampling not supported in this script.
$bNoMissing = false; // only include sites that doesn't have any NAs in any individuals
$nMinNonMissingIndividuals = 1; // min number of required non-NA individuals in order to keep a site, currently set arbitrarily since we already performed coverage filtering
$bOutputExcludedSitesAsNA = true; //keep excluded sites in the output, and output them as NAs

$arrExcludeList = array(); // specify individuals to exclude by their labels.


/* ================= DO NOT CHANGE THINGS BELOW THIS LINE =================== */

$hGenotypeFilePop1 = fopen($sGenotypeFilePop1 , "r");
$hGenotypeFilePop2 = fopen($sGenotypeFilePop2 , "r");
$hDOut = fopen($sDOut, "w");

$arrFilterList = array();



$arrMatrix = array();
$arrMask = array();
$arrExcludeSites = array();
$arrCoordinates = array();
$nLnCount = -1;
$nSampleCount = 0;
echo("Reading genotype file...\n");

while(($sLnPop1 = fgets($hGenotypeFilePop1))!==false && ($sLnPop2 = fgets($hGenotypeFilePop2))!==false) {

	
	$nLnCount++;
	
	if (($nLnCount - 1 )> $nIndividualLimit ) {
		break;
	}
	
	$sLnPop1 = trim($sLnPop1);
	$sLnPop2 = trim($sLnPop2);
	
	$arrFieldsPop1 = explode("\t", trim($sLnPop1));
	$arrFieldsPop2 = explode("\t", trim($sLnPop2));
	
	$nFieldCountPop1 = count($arrFieldsPop1);
	
	if ($nLnCount == 0) { // parse header	
			for($i=0;$i<$nFieldCountPop1;$i++) {
		
			  $sHeaderPop1 = $arrFieldsPop1[$i];
			 
			  preg_match("|(\d+)\:(\d+)|",  trim( $sHeaderPop1 ), $arrScfld);
			  $nScfld = $arrScfld[1];
			  $nCoord = $arrScfld[2];
			  
			  array_push($arrCoordinates , array($nScfld, $nCoord));
			  //print_r($arrCoordinates);
			  //die();
			  //print_r($arrScfld);
			  //die();
			  if ($nScfld < $nScfldLimit) {
				array_push($arrMask , true);
			  }
			  else {
				array_push($arrMask , false);
				//echo( $nScfld." ");
			  }
			  
			  if ( $arrFieldsPop2[$i] != $arrFieldsPop1[$i] )	{
				die("Headers don't match between the three input files at column $i");
			  }
	
			}
			
		echo(count($arrMask)." loci indicated in header".PHP_EOL);
		continue;
	}
	
	
	
	
	if ($sLnPop1 == "" || $sLnPop2 == "") continue;
	
	
	$nFieldCountPop1 = count($arrFieldsPop1);
	$nFieldCountPop2 = count($arrFieldsPop2);
	
	if ($nFieldCountPop1!=$nFieldCountPop2 ) {
		die("The number of fields on line $nLnCount differs between files: Pop1 $nFieldCountPop1 ; Pop2 $nFieldCountPop2 ;");
	}
	
	if ($arrFieldsPop1[0] != $arrFieldsPop2[0]) {
	
		die("Sample labels disagree among the three genotype files on line  $nLnCount. Pop1 ".$arrFieldsPop1[0]." ; Pop2 ".$arrFieldsPop2[0] ." ; ");
	}
	
	
	if (in_array($arrFieldsPop1[0], $arrExcludeList ) ) { // exclude it!
		continue;
	}
	
	$arrIndividual = array();
	
	for($i=1;$i<$nFieldCountPop1;$i++) {
		if (isset($arrMask[$i-1])) {
		
			if ($arrMask[$i-1]) {
				array_push( $arrIndividual , fnParseField($arrFieldsPop1[$i],$arrFieldsPop2[$i]));	
			}
		}
		else {
			//echo($arrFieldsPop1[$i]);
		}
		
		$bIsNa = fnIsNa($arrFieldsPop1[$i],$arrFieldsPop2[$i]);

		if (!isset($arrExcludeSites[$i-1])) {
			$arrExcludeSites[$i-1] = false ;
		}
			
		if ($bIsNa && $bNoMissing) {
			$arrExcludeSites[$i-1] = true;
		}
	}
	
	if ( trim($arrFieldsPop1[0]) != "") {
		$sSampleName = str_replace("-", "", trim($arrFieldsPop1[0])); //remove all "-"
		$arrMatrix[$sSampleName] = $arrIndividual;
	}
	
	$nSampleCount++;

}

//print_r($arrMatrix);


$arrIndividualNames = array_keys($arrMatrix);
$nLoci = count($arrMatrix[$arrIndividualNames[0]]);

echo("Parsed in ".count($arrIndividualNames )." individuals and $nLoci loci".PHP_EOL);


$nPrevScfld = -1;
$nPrevCoord = -1 - $nSubSampleEvery;
$nThresholdCoord = $nPrevCoord + $nSubSampleEvery;


//print_r($arrCountAllele1);
echo("Calculating Pairwise D and r-square ...\n");

$arrMatrixD = array();
$arrMatrixDprime = array();
$arrRsquare = array();
$arrMatrixNumInd = array();

fnWriteHeader();
for ($i=0;$i<$nLoci;$i++) {
	
	$arrMatrixD[$i] = array();
	$arrMatrixDprime[$i] = array();
	$arrRsquare[$i] = array();
	$arrMatrixNumInd[$i] = array();
	
	for($j=$i;$j<$nLoci;$j++) {
		
		if ($arrExcludeSites[$i] || $arrExcludeSites[$j]) { // excluded site, no need to calculate
		
			fnWriteOut($i, $j , "NA", "NA", "NA", "NA");
			continue;
		}
		
		$arrY = array();
		$arrZ = array();
		$nPrevYVal = $nPrevZVal = 0;
		$bYValPolyMorphic = $bZValPolyMorphic = false;
		$bIsFirst = true;
		
		foreach($arrMatrix as $arrInv) {
			if ($arrInv[$i] == -1 || $arrInv[$j] == -1) {
			
				continue;
			}
			
			if ($bIsFirst) {
				$nPrevYVal = $arrInv[$i];
				$nPrevZVal = $arrInv[$j];
				$bIsFirst = false;
			}
			
			if (!$bYValPolyMorphic) {
				if ($nPrevYVal != $arrInv[$i]) {
					$bYValPolyMorphic = true;
				}
			}
			
			if (!$bZValPolyMorphic) {
				if ($nPrevZVal != $arrInv[$j]) {
					$bZValPolyMorphic = true;
				}
			}
			
			array_push($arrY ,$arrInv[$i] );
			array_push($arrZ ,$arrInv[$j] );
			
			
		}
		
		if (!$bYValPolyMorphic) {
			$arrExcludeSites[$i] = true;
		}
		
		if (!$bZValPolyMorphic) {
			$arrExcludeSites[$j] = true;
		}
		
		if (!$bYValPolyMorphic || !$bZValPolyMorphic) { // one of the locus is fixed.
			fnWriteOut($i, $j , "NA", "NA", "NA", count($arrY));
			continue;
		}
		


		if (count($arrY) < $nMinNonMissingIndividuals) {
			fnWriteOut($i, $j , "NA", "NA", "NA", count($arrY));
			continue;
		}
		
		//print_r($arrZ);
		//die();
		
		
		list($pA, $pB, $nD , $nDPrime, $nRsquare) = get_covD($arrY , $arrZ );
    
		
		fnWriteOut($i, $j , $nD, $nDPrime, $nRsquare, count($arrY));
		
		
		
	}
}

function fnWriteHeader() {
	global $hDOut;
	fwrite($hDOut , "Chr1\tChr1Coord\tChr2\tChr2Coord\tD\tDPrime\tR\tIncludedInd".PHP_EOL);
}
function fnWriteOut($i, $j , $nD, $nDPrime, $nRsquare, $nInd) {
global $arrCoordinates, $bOutputExcludedSitesAsNA, $arrExcludeSites, $hDOut;
		$sHeaderStr="";
		$nCurrScfld1 = $arrCoordinates[$i][0];
		$nCurrCoord1 = $arrCoordinates[$i][1];
		
		$nCurrScfld2 = $arrCoordinates[$j][0];
		$nCurrCoord2 = $arrCoordinates[$j][1];
		
		if ($arrExcludeSites[$i] && (!$bOutputExcludedSitesAsNA)) {
			continue;
		}
		
		$sHeaderStr .= $nCurrScfld1."\t".$nCurrCoord1."\t".$nCurrScfld2."\t".$nCurrCoord2."\t";
	

	fwrite($hDOut , "$sHeaderStr$nD\t$nDPrime\t$nRsquare\t$nInd".PHP_EOL);
	//fwrite($hDprimeOut , $sHeaderStr.$nDPrime.PHP_EOL);
	//fwrite($hRsquareOut , $sHeaderStr.$nRsquare.PHP_EOL);
	//fwrite($hNumIndividualOut , $sHeaderStr.$nInd.PHP_EOL);

}


function fnParseField($nProbPop1, $nProbPop2) {
	global $nCutoffProb;
	if (strtoupper($nProbPop1)=="NA" || strtoupper($nProbPop2) == "NA") {
		//echo("missing1");
		return -1;
	}
	
	
	if (((float)$nProbPop1) >= $nCutoffProb) {
		return 0;
	}
	
	if (((float)$nProbPop2) >= $nCutoffProb) {
		return 2;
	}
	
	$nProbHetero = 1.0 - $nProbPop1 - $nProbPop2;
	if (((float)$nProbHetero) >= $nCutoffProb) {
		return 1;
	}
	
	//echo("missing2");
	return -1; //missing
}

function fnIsNa($nProbPop1, $nProbPop2) {
	global $nCutoffProb;
	if (strtoupper($nProbPop1)=="NA" || strtoupper($nProbPop2) == "NA" ) {
		//echo("missing1");
		return true;
	}
	
	else if (strtoupper($nProbPop1)!="NA" && strtoupper($nProbPop2) != "NA") {
		$nProbHetero = 1.0 - $nProbPop1 - $nProbPop2;
		if (((float)$nProbHetero) < $nCutoffProb || ((float)$nProbPop1) < $nCutoffProb || ((float)$nProbPop2) < $nCutoffProb) {
			return true;
		}
	}
	
	else {
		return false;
	}
	
}

function fnStateToStr($nState) {

	switch($nState) {
		case -1 : return "0\t0\t";
		case 0	: return "1\t1\t";
		case 1	: return "1\t2\t";
		case 2	: return "2\t2\t";
		default: return "0\t0\t";
	}
}

function FilterNegatives($val) {
	return $val>=0;
}

function bivmom(&$vec0, &$vec1){
    /*
    Calculate means, variances, the covariance, from two data vectors.
    On entry, vec0 and vec1 should be vectors of numeric values and
    should have the same length.  Function returns m0, v0, m1, v1,
    cov, where m0 and m1 are the means of vec0 and vec1, v0 and v1 are
    the variances, and cov is the covariance.
    */
    $m0 = $m1 = $v0 = $v1 = $cov = 0;
	$n = count($vec0);
	
    assert($n == count($vec1));
	
	for($i=0;$i<$n;$i++){
		$x = $vec0[$i];
		$y = $vec1[$i];
        $m0 += $x;
        $m1 += $y;
        $v0 += $x*$x;
        $v1 += $y*$y;
        $cov += $x*$y;
	}
	
    
    $n = floatval($n);
    $m0 /= $n;
    $m1 /= $n;
    $v0 /= $n;
    $v1 /= $n;
    $cov /= $n;

    $cov -= $m0 * $m1;
    $v0 -= $m0 * $m0;
    $v1 -= $m1 * $m1;
    return array($m0, $v0, $m1, $v1, $cov);
}

// Rogers-Huff 2008
function get_covD(&$Y, &$Z) {
    /*
    get_covD estimates pA, pB, and D w/o info on gametic phase.
    Uses the method of Rogers and Huff 2008.
    */
    list($pA, $v0, $pB, $v1, $cov) = bivmom($Y,$Z);
	//echo(($pA." ". $v0." ". $pB." ".$v1." ". $cov));
	
    $pA = 0.5*$pA;
    $pB = 0.5*$pB;
    $qA=1-$pA;
    $qB=1-$pB;
    $two_1pf = sqrt(($v0*$v1)/($pA*$qA*$pB*$qB)); // # estimates 2(1+f)
	/*
	if ($two_1pf ==0) {
		die(($pA." ". $v0." ". $pB." ".$v1." ". $cov));
	}
	*/
	
    $D = $cov/$two_1pf;
	$Dprime = ($D >=0)? ($D / min($pA*$qB , $qA*$pB)) : ($D / min($pA*$pB , $qA*$qB)) ;
	$r = $cov/sqrt($v0*$v1);
    return array($pA, $pB, $D, $Dprime, $r);
}

// Rogers-Huff 2008
function get_r(&$Y,&$Z) {
    /*
    Estimates r w/o info on gametic phase.  Also works with gametic
    data, in which case Y and Z should be vectors of 0/1 indicator
    variables.
    Uses the method of Rogers and Huff 2008.
    */
    list($mY, $vY, $mZ, $vZ, $cov) = bivmom($Y,$Z);
    return $cov/sqrt($vY*$vZ);
}

//RH-tabulate
function get_r_corr_genotype($Y, $Z) {
    //print "Enter get_r_corr_genotype"
    $arrcount = array(array(0,0,0),array(0,0,0),array(0,0,0));

    //print "A"
    //print "type(Y):", type(Y)
    //print "len(Y):", len(Y)
	$len = count($Y);
    for($i=0;$i<$len;$i++) {
	
        //print i
        //print Y[i]
        //print Z[i]
        $arrcount[$Y[$i]][$Z[$i]] += 1;
	}

    //print "B",

    $sumx = $sumy = $sumxx = $sumyy = $sumxy = 0;
    for($i=0;$i<3;$i++) {
        for($j=0;$j<3;$j++) {
			$sumx += $arrcount[$i][$j]*$i;
			$sumy += $arrcount[$i][$j]*$j;
			$sumxx += $arrcount[$i][$j]*$i*$i;
			$sumyy += $arrcount[$i][$j]*$j*$j;
			$sumxy += $arrcount[$i][$j]*$i*$j;
		}
	}
	/*
    print "sumx=%d sumy=%d sumxx=%d sumyy=%d sumxy=%d" \
        % (sumx, sumy, sumxx, sumyy, sumxy)*/
    // Calculate numerators in integer arithmetic to
    // avoid roundoff.
    
    $nsqr = floatval($len*($len-1));
    $cov = ($len*$sumxy - $sumx*$sumy)/$nsqr;
    $vx = ($len*$sumxx - $sumx*$sumx)/$nsqr;
    $vy = ($len*$sumyy - $sumy*$sumy)/$nsqr;
    $r = $cov/sqrt($vx*$vy);
    //print "get_r_corr_genotype returning:", r
    return $r;
}

?>